﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.8
@EndOfDesignText@
' if in single mode, we raise an event
#Event: SingleClick(ID as String)

Sub Class_Globals
	Private BANano As BANano 'ignore
	Private mEventName As String
	
	' all record IDs with a switch selected or not
	Private AllItems As Map
	Private ItemInfos As List
	' the template of an item box
	Private HTMLTemplate As String
	' the parameters in the html that need to be replaced with the data
	Private HTMLParams As List
			
	Private mTargetID As String	
	Private mSingle As Boolean = False
	
	Private mElement As BANanoElement
	Private mItems As BANanoElement
End Sub

'Initializes the object
Public Sub Initialize(TargetID As String, eventName As String, single As Boolean)
	mTargetID = TargetID.ToLowerCase
	mEventName = eventName.ToLowerCase
	mSingle = single
	
	AllItems.Initialize
	ItemInfos.Initialize
	HTMLParams.Initialize
	
	' get the element that will hold our list
	mElement.Initialize("#" & mTargetID)
	' let's make the target vertical scrollable
	mElement.AddClass("mi-scroll-vertical")
End Sub

' loading all the items
public Sub LoadAllItems(jsonItemInfosStr As String, IDField As String)
	Dim jsonAll As BANanoJSONParser
	jsonAll.Initialize(jsonItemInfosStr)
	
	Dim lst As List = jsonAll.NextArray
	
	AllItems.Clear
	ItemInfos.Clear
	
	For i = 0 To lst.Size - 1
		Dim m As Map = lst.Get(i)
		Dim ID As Int = m.Get(IDField)
		AllItems.Put(ID, 0)
		ItemInfos.Add(m)
	Next	
End Sub

' To pre-select some items.  e.g. the result of a previous GetAllSelected() call
Public Sub SelectItems(jsonSelectedStr As String)
	Dim jsonP As BANanoJSONParser
	jsonP.Initialize(jsonSelectedStr)
	Dim m As Map = jsonP.NextObject
	
	' reset all to not selected
	For Each key As Int In AllItems.keys
		AllItems.Put(key, 0)		
	Next
	If mItems Then
		mItems.RemoveClass("mi-selected")
	End If	
	
	For Each key As Int In m.Keys
		AllItems.Put(key, 1)
	Next	
End Sub

' Sets a html template to be used by an item
public Sub SetTemplate(template As String, Params As List)
	HTMLTemplate = template
	HTMLParams = Params	
End Sub

' Get all the selected items
public Sub GetAllSelected() As String
	Dim ret As Map
	ret.Initialize
	For Each key As Int In AllItems.keys
		If 1 = AllItems.Get(key) Then
			ret.Put(key, 1)
		End If
	Next
	Return BANano.ToJson(ret)
End Sub

' Build the multi list
public Sub Build(MaxCols As Int)
	Log("Building...")
	
	' add a new row
	Dim NewHTML As String = $"<div class="row mi-row-no-margin-bottom">"$
	For i = 0 To ItemInfos.Size - 1
		' every MaxCols, we add a new row (not for the first one as we did that already)
		If (i > 0) And (i Mod MaxCols = 0) Then
			NewHTML = NewHTML & $"</div><div class="row mi-row-no-margin-bottom">"$
		End If
		
		' get our info
		Dim info As Map = ItemInfos.Get(i)
		' get our html template
		Dim tmpTemplateInfo As String = HTMLTemplate
		' replace the eventName
		tmpTemplateInfo = tmpTemplateInfo.Replace("{eventName}", mEventName)
		' replace all the parameters
		For j = 0 To HTMLParams.Size - 1			
			tmpTemplateInfo = tmpTemplateInfo.Replace("{" & HTMLParams.Get(j) & "}", info.Get(HTMLParams.Get(j)))
		Next		
		' if the item is selected
		If 1 = AllItems.Get(info.Get("id")) Then
			tmpTemplateInfo = tmpTemplateInfo.Replace("{selected}", " mi-selected ")
		Else
			tmpTemplateInfo = tmpTemplateInfo.Replace("{selected}", "")
		End If
		NewHTML = NewHTML & tmpTemplateInfo		
	Next
	' close the last row
	NewHTML = NewHTML & "</div>"	
	
	' add our html
	mElement.SetHTML(NewHTML)
	
	' add a click event to each item
	mItems.Initialize(".mi-box-" & mEventName	)
	mItems.Off("click").On("click", Me, "HandleClick")
End Sub

' empty the list
public Sub Empty()	
	If mTargetID <> "" Then
		mElement.SetHTML("")
	End If	
End Sub

' handle the click on an item
private Sub HandleClick(event As BANanoEvent) 'ignore
	Dim ID As BANanoObject = event.Target
	Dim elem As BANanoElement
	
	' get the item box
	elem.Initialize("#" & ID.GetField("id").Result)
	
	' remove the 'ms' prefix so we get the record id
	Dim IDStr As String = "" & ID.GetField("id").Result
	IDStr = IDStr.SubString(2)
	
	If mSingle Then
		' select the item 
		elem.AddClass("mi-selected")
		' mark it in our total item list
		AllItems.Put(IDStr, 1)
		' raise an event to ABM 
		BANano.RaiseEventToABM(mEventName & "_singleclick", Array("ID"), Array(IDStr), "")		
	Else
		' check if it is selected
		If elem.HasClass("mi-selected") Then
			' remove the selection
			elem.RemoveClass("mi-selected")
			' mark it in our total item list
			AllItems.Put(IDStr, 0)
		Else
			elem.AddClass("mi-selected")
			AllItems.Put(IDStr, 1)			
		End If
	End If		
End Sub

#if CSS
.mi-box {
    border: 1px solid lightgrey;
    cursor: pointer;
    border-radius: 5px;
    border-left: 9px solid lightgrey;
	margin-top: 6px;
	margin-bottom: 6px;
	opacity: 0.7;
	-webkit-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	-o-user-select: none;
	user-select: none;
}
.mi-box.mi-selected {
	border: 2px solid #0D4C5E;
    border-left: 9px solid #0D4C5E;
	background-color: #E6EFEE;
	opacity: 1;
	box-shadow: 0 2px 5px 0 rgb(0 0 0 / 16%), 0 2px 10px 0 rgb(0 0 0 / 12%);
	margin-top: 5px;
	margin-bottom: 5px;
}
.mi-row {
	padding-left: 5px;
    padding-right: 5px;
    margin-bottom: 5px;
    margin-top: 5px;
	pointer-events: none;
}
.mi-row-no-margin-right {
	margin-right: 0px !important;
}
.mi-row-no-margin-bottom {
	margin-bottom: 0px !important;
}
.mi-col {
	height: 1.3em;
    pointer-events: none;
    overflow: hidden;
    text-overflow: ellipsis;
    display: block;
    max-height: 1.3em;
    line-height: 1.3em;
    display: -webkit-box;
    -webkit-line-clamp: 1;
    -webkit-box-orient: vertical;
}
.mi-col-smaller {
	font-size: smaller;
}
.mi-col-bold {
	font-weight: 600;
}
.mi-col-left {
	float: left;
	margin-top: 5px;
}
.mi-col-right {
	display: table;
	padding-top: 4px;
}
.mi-scroll-vertical {
	overflow: hidden auto;
}
.mi-img {
	margin-left: 10px;
    border-radius: 4px;
    border: 1px solid lightgrey;	
}
#End If